from urllib.parse import urljoin

from flask import abort, jsonify, redirect, render_template, request

from app import app
from db import get_url, write_url
from utils import get_expiry, get_random_name, return_json
from validation import (
    ValidationError,
    validate_expiry,
    validate_name,
    validate_url,
)


def home_page():
    if return_json():
        return jsonify("Hi, I'm abbr. "
                       "Usage: POST either a url as string or a dict with "
                       "'url' and optionally 'name' and 'expiry' "
                       "('%Y-%m-%d %H:%M:%S').")
    return render_template('index.html')


def create_short_url():
    if len(request.form) == 1 and 'url' not in request.form.keys():
        url = next(request.form.keys())
    else:
        url = request.form.get('url')
    name = request.form.get('name') or get_random_name()
    expiry = request.form.get('expiry') or get_expiry()

    try:
        url = validate_url(url)
        name = validate_name(name)
        expiry = validate_expiry(expiry)
    except ValidationError as err:
        message = err.message
    else:
        write_url(name, url, expiry)
        message = urljoin(app.config.get('DOMAIN'), name)

    if return_json():
        return jsonify(message)
    return render_template('index.html', message=message)


@app.route('/', methods=['GET', 'POST'])
def main():
    if request.method == 'GET':
        return home_page()
    elif request.method == 'POST':
        return create_short_url()


def get_redirect_or_404(short_url):
    url = get_url(short_url)
    if not url:
        return abort(404)
    return redirect(url, code=301)


def get_info_or_404(short_url):
    url = get_url(short_url)
    if not url:
        return abort(404)
    detail = {'name': short_url, 'url': url}
    if return_json():
        return jsonify(detail)
    return render_template('detail.html', **detail)


@app.route('/<short_url>', methods=['GET'])
def get_redirect(short_url):
    if 'info' in request.args:
        return get_info_or_404(short_url)
    else:
        return get_redirect_or_404(short_url)



import os
from flask import Flask


app = Flask(__name__, instance_relative_config=True)

app.config.from_object('config.Config')
if os.environ.get('ABBR_CONFIG'):
    app.config.from_envvar('ABBR_CONFIG')



import os
import string
from datetime import timedelta


class Config:
    """ The default config class for abbr. """

    # The fqdn where your abbr is hosted
    SITENAME = 'abbr.example.com'
    DOMAIN = 'https://{}'.format(SITENAME)

    # Defautl expiry offset
    EXPIRY = timedelta(days=365)

    # Default short_url length
    NAME_LENGTH = 8

    # Default short_name alphabet
    ALPHABET = string.ascii_lowercase + string.digits

    # Default database settings for sqlite
    DATABASE = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        'abbr.db',
    )

    SECRET_KEY = 'very secret key'
    USERNAME = 'abbr-admin'
    PASSWORD = 'abbr-password'



from codecs import open
from os import path
from setuptools import setup

setup(
    name = 'abbr',
    version = '0.0.0',

    description = 'Self-hosted URL shortener',

    url = 'https://github.com/rixx/abbr',

    author = 'Tobias Kunze',
    author_email = 'rixx@cutebit.de',

    license = 'Apache Software License',

    classifiers = [
        'License :: OSI Approved :: Apache Software License',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    packages = ['abbr'],

    install_requires = [
        'Flask==0.11.1',
    ],
)



import re
from datetime import datetime
from urllib.parse import urlparse

from app import app


class ValidationError(Exception):
    def __init__(self, message, *args, **kwargs):
        self.message = message
        super().__init__(message, *args, **kwargs)


def validate_expiry(expiry):
    try:
        return datetime.strptime(expiry, '%Y-%m-%d %H:%M:%S')
    except:
        raise ValidationError(
            {'expiry': 'Expiry needs to be in format %Y-%m-%d %H:%M:%S.'}
        )


def validate_name(name):
    if len(name) < 2:
        raise ValidationError(
            {'name': 'Name needs to be at least 2 characters long.'}
        )
    if len(name) > 30:
        raise ValidationError(
            {'name': 'This is supposed to be a *shortening* service. '
                     'Names need to be at most 30 characters long.'}
        )
    if not re.match('^[{}]*$'.format(app.config.get('ALPHABET')), name):
        raise ValidationError(
            {'name': 'A name may only contain the following characters: {}'
                     .format(app.config.ALPHABET)}
        )
    return name


def validate_url(url):
    if url[0] == '"' or url[0] == "'":
        url = url[1:-1]

    _url = urlparse(url, scheme='http')
    # this is horrid validation. should really use Django
    if not (_url.netloc or _url.path):
        raise ValidationError({'url': 'This is not a valid URL.'})
    return url



from datetime import datetime
import random

from flask import request

from app import app


def return_json():
    if 'curl' in request.headers.get('User-Agent'):
        return True
    if 'wget' in request.headers.get('User-Agent'):
        return True
    best = request.accept_mimetypes.best_match(
        ['application/json', 'text/html']
    )
    return best == 'application/json' and \
        request.accept_mimetypes[best] > request.accept_mimetypes['text/html']


def get_random_name():
    import db
    conn = db.get_db_connection()
    i = 10

    while i > 0:
        name = ''.join(
            random.choice(app.config.get('ALPHABET'))
            for _ in range(app.config.get('NAME_LENGTH'))
            )
        query = conn.execute('SELECT * FROM urls WHERE name = ?', (name, ))
        if not query.fetchall():
            return name
    raise Exception('Unable to find unique name.')


def get_expiry():
    then = datetime.now() + app.config.get('EXPIRY')
    return then.strftime('%Y-%m-%d %H:%M:%S')



import sqlite3
from contextlib import closing

from flask import g

from app import app


def connect_db():
    connection = sqlite3.connect(app.config['DATABASE'])
    connection.row_factory = sqlite3.Row
    return connection


def get_db_connection():
    if not hasattr(g, 'db_connection'):
        g.db_connection = connect_db()
    return g.db_connection


@app.before_request
def before_request():
    g.db_connection = get_db_connection()


@app.teardown_appcontext
def close_db_connection(exception):
    if hasattr(g, 'db_connection'):
        g.db_connection.close()


def init_db():
    with closing(connect_db()) as db:
        with app.open_resource('schema.sql', 'r') as schema_file:
            db.cursor().executescript(schema_file.read())
        db.commit()


@app.cli.command('initdb')
def command_initdb():
    init_db()
    print('Initialized the database')


def get_url(short_name):
    conn = get_db_connection()
    cursor = conn.cursor()
    print(short_name)
    cursor.execute(
        'SELECT url FROM urls WHERE name = ?',
        (short_name, ),
    )
    result = cursor.fetchall()
    if len(result) != 1:
        return None
    return result[0]['url']


def write_url(short_name, url, expiry):
    conn = get_db_connection()
    conn.execute(
        'INSERT INTO urls (name, url, expiry) values (?, ?, ?)',
        (short_name, url, expiry),
    )
    conn.commit()
